\name{RaProR-package}
\alias{RaProR-package}
\alias{RaProR}
\docType{package}
\title{
Create a smaller substitute data set to perform linear regression
}
\description{
This package can be used to calculate sketches of a data set that can be used to perform approximate classical or Bayesian linear regression. The sketch is a substitute data set of the same dimension but much smaller number of observations. The inference based on the sketch is much faster and is provably close to the exact inference. The calculation is done time- and space-efficiently via a C++ function. The two main functions are \code{\link{sketch}} for data sets that fit into the working memory and can be processed at once and \code{\link{readinandsketch}} for data sets that (potentially) do not fit into the working memory and will be read and sketched sequentially blockwise.
}
\details{
\tabular{ll}{
Package: \tab RaProR\cr
Type: \tab Package\cr
Version: \tab 1.0\cr
Date: \tab 2015-07-14\cr
License: \tab GPL (>= 3)\cr
}
}
\author{
LN Geppert, K. Ickstadt, A. Munteanu, J. Quedenfeld, C. Sohler
}
\references{
Geppert, LN, Ickstadt, K., Munteanu, A., Quedenfeld, J., Sohler, C. (2015). Random projections for Bayesian regression. stat.CO, abs/1504.06122, \url{http://arxiv.org/abs/1504.06122}	
}
% ~~ Optionally other standard keywords, one per line, from file KEYWORDS in ~~
% ~~ the R documentation directory ~~
\keyword{Linear regression}
\keyword{Bayesian linear regression}
\keyword{Random projections}
\keyword{Data reduction}
\keyword{Big data}
\examples{
# create a small simulated data set
# with 400 observations and
# 4 variables
set.seed(23)
x1 = rnorm(400, 10, 2)
x2 = rnorm(400, 5, 3)
x3 = rnorm(400, -2, 1)
x4 = rnorm(400, 0, 5)
y = 2.4 - 0.6 * x1 + 5.5 * x2 - 7.2 * x3 + 5.7 * x4 + rnorm(400)
# all in one data.frame
data = data.frame(x1, x2, x3, x4, y)

# linear model based on original data set
lm(y~., data = data)

# Calculate an RAD/"R"-sketch with epsilon = 0.2
s1 = sketch(data, epsilon = 0.2, method = 'R', affine = TRUE)
dim(s1)
lm(y~.-1, data = s1)
}
