\name{setSeed}
\alias{setSeed}
\title{
Set the seed for the random number generator
}
\description{
Sets the seed for the random number generator that is used by \code{\link{sketch}} and \code{\link{readinandsketch}}.
}
\usage{
setSeed(seed)
}
\arguments{
  \item{seed}{
    Seed for the random number generator.
	}
}
\details{
By default, the seed is initialized using the current time. Setting the seed before calculating the first sketch assures reproducibility. Calling \code{\link{sketch}} or \code{\link{readinandsketch}} with the same data set or file after setting the seed to a certain value will result in the same sketch. This is not the case (with high probability) otherwise.
}
\value{
	This function is used for its side-effects and does not return a value.
}
\references{
Geppert, LN, Ickstadt, K., Munteanu, A., Quedenfeld, J., Sohler, C. (2015). Random projections for Bayesian regression. stat.CO, abs/1504.06122, \url{http://arxiv.org/abs/1504.06122}
}
\author{
LN Geppert, K. Ickstadt, A. Munteanu, J. Quedenfeld, C. Sohler
}

\seealso{
\code{\link{sketch}}
\code{\link{readinandsketch}}
}
\examples{
# create a small simulated data set
# with 400 observations and
# 4 variables
set.seed(23)
x1 = rnorm(400, 10, 2)
x2 = rnorm(400, 5, 3)
x3 = rnorm(400, -2, 1)
x4 = rnorm(400, 0, 5)
y = 2.4 - 0.6 * x1 + 5.5 * x2 - 7.2 * x3 + 5.7 * x4 + rnorm(400)
# all in one data.frame
data = data.frame(x1, x2, x3, x4, y)

# linear model based on original data set
lm(y~., data = data)

# Calculate an RAD/"R"-sketch with epsilon = 0.2
setSeed(5)
s1 = sketch(data, epsilon = 0.2, method = 'R', affine = TRUE)
dim(s1)
head(s1)
lm(y~.-1, data = s1)

# same sketch again
setSeed(5)
s1 = sketch(data, epsilon = 0.2, method = 'R', affine = TRUE)
dim(s1)
head(s1)
lm(y~.-1, data = s1)

# different sketch, but similar results for the regression model
s1 = sketch(data, epsilon = 0.2, method = 'R', affine = TRUE)
dim(s1)
head(s1)
lm(y~.-1, data = s1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{Linear regression}
\keyword{Bayesian linear regression}
\keyword{Random projections}
\keyword{Data reduction}
\keyword{Big data}
