/** @file Statistik.cpp
 *  @brief Implementation of Statistik.h
 *  @author Jens Quedenfeld
 */

#include "exp/Statistik.h"
#include "sketch/BCH.h"
#include "sketch/SRHT_direct.h"
#include "util/Useful.h"
#include <cmath>
#include <cstdlib>
#include <cstdio>
#include <ctime>
#include <iostream>
#include <map>

#if __cplusplus >= 201103L
#include <unordered_map>
#endif

using namespace statistik;
using namespace std;
using namespace useful;

Readed statistik::read(const char* file_Ab, const char* file_x) {
	long long t0 = clock();
	Readed erg;
	erg.name = string(file_Ab);

	// Datei �ffnen
	ifstream f(file_Ab);
	if (!f) err("error: read_fast: f.open()");
	string line;

	// dim2 berechnen (beachte, dass die letzten beiden Spalten nicht dazugeh�ren! )
	size_t begin = 0;
	int dim2 = -2;
	getline(f, line);
	while (begin != string::npos){
		dim2++;
		begin = line.find_first_of(' ', begin + 1);
	}

	// Datensammel-Vektoren
	vector<vector<double> > data_A;
	vector<double> data_b;

	// Daten lesen
	getline(f, line);
	int i = 0;
	while (!f.eof()) {
		data_A.push_back(vector<double>(dim2));
		char* c = &line[0];
		for (int j = 0; j < dim2 + 1; j++){
			double entry = strtod(c, &c);
			if (j == dim2){
				data_b.push_back(entry);
			} else {
				data_A[i][j] = entry;;
			}
		}
		getline(f, line);
		i++;
	}
	int dim1 = i;

	// Daten in Matrizen schreiben
	erg.A = Matrix<double>(dim1, dim2);
	erg.b = Matrix<double>(dim1, 1);
	for (int i = 0; i < dim1; i++) {
		for (int j = 0; j < dim2; j++) {
			erg.A.updateEntry(i, j, data_A[i][j]);
		}
		erg.b.updateEntry(i, 0, data_b[i]);
	}

	// Betadaten auslesen
	if (file_x != NULL) {
		ifstream g(file_x);
		if (!g) err("error: readtxt: g.open()");
		getline(g, line);
		erg.x = Matrix<double>(dim2, 1);
		for (int i = 0; i < dim2; i++) {
			getline(g, line);
			double entry = atof(line.data());
			erg.x.updateEntry(i, 0, entry);
		}
	} else {
		// Leere x-Matrix, falls Betadaten nicht vorhanden
		erg.x = Matrix<double>();
	}

	// Laufzeit bestimmen
	erg.time = clock() - t0;
	return erg;
}


Readed statistik::read_slow(const char* filename_1, const char* filename_2) {
	long long t0 = clock();
	Readed erg;
	erg.name = string(filename_1);

	ifstream f;
	string line;

	f.open(filename_1, ios::in);
	if (!f) err("error: readtxt: f.open()");

	int dim1, dim2;
	getline(f, line);

	// Dim1 ermitteln (entspricht Anzahl der Zeilen - 1)
	dim1 = -1;
	while (!f.eof() ){
		dim1++;
		getline(f, line);
	}

//	f.close();
//
//	f.open(filename_1, ios::in);
	f.clear();
	f.seekg(0, ios_base::beg);
	if (!f) err("error: readtxt,2: f.seekg()");

	size_t begin, end;
	getline(f, line);
	getline(f, line);

	// dim2 berechnen (beachte, dass die letzten beiden Spalten nciht dazugeh�ren! )
	begin = 0;
	dim2 = -2;
	while (begin != string::npos){
		dim2++;
		begin = line.find_first_of(' ', begin + 1);
	}

	// Betadaten auslesen
	if (filename_2 != NULL) {
		fstream g;
		string line_g;
		g.open(filename_2, ios::in);
		if (!g) err("error: readtxt: g.open()");
		getline(g, line_g);
		getline(g, line_g);

		erg.x = Matrix<double>(dim2, 1);
		for (int i = 0; i < dim2; i++){
			double entry = atof(line_g./*substr(0, 25).*/data() );
			erg.x.updateEntry(i, 0, entry);
			if (i != dim2 - 1)
				getline(g, line_g);
		}
	} else {
		// Leere x-Matrix, falls Betadaten nicht vorhanden
		erg.x = Matrix<double>();
	}

	// Erstellen der Matrizen f�r die Ergebnis-Struct
	erg.A = Matrix<double>(dim1, dim2);
	erg.b = Matrix<double>(dim1, 1);

	// Daten f�r A und b
	for (int i = 0; i < dim1; i++){
		begin = 0;
		for (int j = 0; j < dim2 + 1; j++){
			end = line.find_first_of(' ', begin);
			double entry = atof(line.substr(begin, end - begin).data() );
			if (j == dim2){
				erg.b.updateEntry(i, 0, entry);
			}
			else{
				erg.A.updateEntry(i, j, entry);
			}
			begin = end + 1;
		}
		if (i != dim1 - 1)
			getline(f, line);
	}

	// Laufzeit bestimmen
	erg.time = clock() - t0;
	return erg;
}

void statistik::write(const Base& daten, const char* filename){
	fstream f;
	f.open(filename, ios::out);
	//Schreiben der ersten Zeile
	for (int i = 0; i < (daten.A).getDim2(); i++ ){
		f << "\"x" << i << "\"" << " ";
	}

	f << "\"y\" \"fehler\"";
	f << '\n';

	//Schreiben der Werte
	for (int i = 0; i < (daten.A).getDim1(); i++){
		//Schreiben von A
		for (int j = 0; j < (daten.A).getDim2(); j++){
			f << (daten.A).getEntry(i, j) << " ";
		}
		//Schreiben von b
		f << (daten.b).getEntry(i, 0);

		//Zeilenumbruch, falls Ende der Daten noch nicht erreicht
		if (i != (daten.A).getDim1() - 1)
			f << '\n';
	}

	f << flush;
	f.close();
}

//namespace {
//	double pos(double x) {
//		return x < 0 ? 0 : x;
//	}
//}

Analysed statistik::analyse(const Matrix<double>& matA) {
	Matrix<double> b = Matrix<double>::zero(matA.getDim1(), 1);
	return analyse(matA, b, 0);
}
Analysed statistik::analyse(const Base& base) {
	return analyse(base.A, base.b, base.time);
}
Analysed statistik::analyse(const Matrix<double>& matA, const Matrix<double>& matb, long long time_base) {
	long long t0 = clock();
	Analysed erg;
//	int n = matA.getDim1();
//	int d = matA.getDim2();

	// var
//	erg.var = Matrix<double>::initATA(d);
//	for (int i = 0; i < n; i++) {
//		vector<double> v(d);
//		for (int j = 0; j < d; j++)
//			v[j] = matA(i, j);
//		erg.var.updateATA(&v[0]);
//	}
//	erg.var = erg.var.pseudo_inv();
	erg.var = matA.calcATA().pseudo_inv();

	// mean
//	Matrix<double> atb = Matrix<double>::initATb(d);
//	vector<double> vecb = matb.toVector();
//	for (int i = 0; i < n; i++) {
//		vector<double> v(d);
//		for (int j = 0; j < d; j++)
//			v[j] = matA(i, j);
//		atb.updateATb(&v[0], &vecb[0], i);
//	}
//	erg.mean = erg.var * atb;
	erg.mean = erg.var * (matA.getTranspose() * matb);

	// eigen
	eigenData<double> ed = erg.var.eigen(Matrix<double>::SVD_CPPL);
	erg.eigenValues = Matrix<double>::fromVectorRow(ed.values);
	erg.eigenValues.transform(fabs);
	erg.eigenValues.transform(sqrt);
	erg.eigenVectors = ed.vectors;

	erg.A.set_noCopy(matA); //.select(0, 0, matA.getDim1(), matA.getDim2())); // Nur implizite Kopie!
	erg.b.set_noCopy(matb); //.select(0, 0, matb.getDim1(), matb.getDim2())); // Nur implizite Kopie!
	erg.time_analysis = clock() - t0;
	erg.time_base = time_base;
	return erg;
}

int statistik::calcDim_Reduced(double eps, int d) {
	return (int) ceil(d * log(d) / (eps * eps));
}

vector<double> statistik::leverageScores_approx(const Matrix<double>& matA, double eps) {
	int n = matA.getDim1();
	int d = matA.getDim2();
    int s1 = (int) ceil(d * log(d) / (eps * eps)); //(d * log(n) / eps / eps * log(d * log(n) / eps / eps) + 1);
    int s2 = (int) ceil(log(n) / (eps * eps));

	// SRHT-Skizze erstellen
	//if (s1 > n) cout << "s1 > n" << endl;
	const Matrix<double>& matSA = s1 > n ? matA : embed<SRHT>(matA, s1);

	// Singul�rwerte und V-Matrix berechnen
	eigenData<double> ed = matSA.calcATA().eigen(Matrix<>::SVD_BEST); // liefert S und V

	// R_inv = V*S_inv
	vector<double> s_inv(ed.values.size());
	double zero_eps = 0;
	for (unsigned i = 0; i < s_inv.size(); i++)
		zero_eps += s_inv[i] = sqrt(fabs(ed.values[i]));
	zero_eps = zero_eps / s_inv.size() / Matrix<>::ACC_PINV_ZERO_DIV;
	for (unsigned i = 0; i < s_inv.size(); i++)
		s_inv[i] = s_inv[i] < zero_eps ? 0 : 1 / s_inv[i];
	Matrix<double> r_inv = ed.vectors.multiplyWithDiagonal(s_inv);

	// U-Matrix ggf. über Skizze berechnen (U = A*R_inv)
	Matrix<double> u = s2 > d ? matA * r_inv : matA * embed<BCH_Sketch>(r_inv.getTranspose(), s2).getTranspose();

	// LeverageScores berechnen
	vector<double> leverageScores(n);
	for (int i = 0; i < n; i++) {
		leverageScores[i] = u.select(i, 0, 1, u.getDim2()).frobeniusnorm(true);
	}

	return leverageScores;
}

vector<double> statistik::leverageScores_exact(const Matrix<double>& a) {
	Matrix<> u = a.svd_small().u;
	vector<double> l(a.getDim1());
	for (int i = 0; i < a.getDim1(); i++)
		l[i] = u.select(i, 0, 1, a.getDim2()).frobeniusnorm(true);
	return l;
}

namespace {
	double min(double a, double b) {
		return a < b ? a : b;
	}
	double max(double a, double b) {
		return a > b ? a : b;
	}
	struct eigenPair {
		double eigenValue;
		int eigenVectorID;
		eigenPair() : eigenValue(0.0), eigenVectorID(0) {}
		eigenPair(double val, int id) : eigenValue(val), eigenVectorID(id) {}
	};
	bool eigenSort(eigenPair a, eigenPair b) {
		return a.eigenValue > b.eigenValue;
	}
	void calcWinkel(const Analysed& exakt, const Analysed& skizze, Epsilon& output) {

		int eigenVectorAnzahl = exakt.eigenVectors.getDim2();

		// winkel_original
		output.angles_original = Matrix<>(1, eigenVectorAnzahl);
		for (int j = 0; j < eigenVectorAnzahl; j++) {
			output.angles_original(0, j) = 0;
			for (int i = 0; i < exakt.eigenVectors.getDim1(); i++) {
				output.angles_original(0, j) += exakt.eigenVectors(i,j) * skizze.eigenVectors(i,j);
			}
		}

		// winkel_greedy
		output.angles_greedy = Matrix<>(1, eigenVectorAnzahl);
		vector<int> vec(eigenVectorAnzahl);
		for (unsigned i = 0; i < vec.size(); i++)
			vec[i] = i;
		for (int j = 0; j < eigenVectorAnzahl; j++) { // w�hle j-ten Eigenvektor von "exakt"
			double max_skal = 0;
			int max_id = -1;

			// Besten Vektor finden
			int k;
			for (k = j; k < eigenVectorAnzahl; k++) { // w�hle (k-j)-ten noch nicht benutzten Eigenvektor von "skizze"
				// Berechne das Skalarprodukt
				double d = 0;
				for (int i = 0; i < exakt.eigenVectors.getDim1(); i++) {
					d += exakt.eigenVectors(i,j) * skizze.eigenVectors(i,vec[k]);
				}
				// Speichere betragsm��ig maximales Skalarprodukt
				if (fabs(d) > fabs(max_skal)) {
					max_skal = d;
					max_id = k;
				}
			}

			// Speichere maximales Skalarprodukt
			output.angles_greedy(0, j) = max_skal;

			// max_id wurde benutzt => Durch j ersetzen
			vec[max_id] = vec[j];
		}

		// winkel_sortiert
		vector<eigenPair> eigen_exakt(eigenVectorAnzahl);
		for (int i = 0; i < eigenVectorAnzahl; i++) {
			eigen_exakt[i] = eigenPair(exakt.eigenValues(0, i), i);
		}
		sort(eigen_exakt.begin(), eigen_exakt.end(), eigenSort);

		vector<eigenPair> eigen_skizze(eigenVectorAnzahl);
		for (int i = 0; i < eigenVectorAnzahl; i++) {
			eigen_skizze[i] = eigenPair(skizze.eigenValues(0, i), i);
		}
		sort(eigen_skizze.begin(), eigen_skizze.end(), eigenSort);

		output.angles_sorted = Matrix<>(1, eigenVectorAnzahl);
		for (int j = 0; j < eigenVectorAnzahl; j++) {
			output.angles_sorted(0, j) = 0;
			for (int i = 0; i < exakt.eigenVectors.getDim1(); i++) {
				output.angles_sorted(0, j) += exakt.eigenVectors(i, eigen_exakt[j].eigenVectorID)
						* skizze.eigenVectors(i, eigen_skizze[j].eigenVectorID);
			}
		}
	}
	void calcWasserstein(const Analysed& exakt, const Analysed& skizze, Epsilon& output) {
		// Wassersteindistanz
		output.trace_var_exact = exakt.var.trace(); // M1 = (A^T*A)+
		output.trace_var_approx = skizze.var.trace(); // M2 = ([SA]^T*SA)+
		Matrix<> m1h = exakt.eigenVectors .multiplyWithDiagonal(exakt.eigenValues.toVector()) * exakt.eigenVectors.getTranspose(); // M1^.5 = (alternativ) U*S^.5*VT
		Matrix<> m1h_m2_m1h =  m1h * skizze.var * m1h; // = M1^.5 * M2 * M1^.5
		svdData<> m1h_m2_m1h_svd = m1h_m2_m1h.svd(); // Berechnung von ^.5 mit der SVD
		for (unsigned i = 0; i < m1h_m2_m1h_svd.s.size(); i++)
			m1h_m2_m1h_svd.s[i] = sqrt(m1h_m2_m1h_svd.s[i]);
		Matrix<> m1h_m2_m1h_h = m1h_m2_m1h_svd.u.multiplyWithDiagonal(m1h_m2_m1h_svd.s) * m1h_m2_m1h_svd.vt; // = (M1^.5 * M2 * M1^.5)^.5
		output.trace_var_mix = m1h_m2_m1h_h.trace(); // mix = [M1^.5*M2*M1^.5]^.5
		output.trace_sum = output.trace_var_exact + output.trace_var_approx - 2*output.trace_var_mix;
	}

	double frobeniusnorm_b_minus_Ax(const Matrix<double>& b, const Matrix<double>& matA, const Matrix<double>& x, bool square = false) {
		if (b.getDim1() != matA.getDim1() || b.getDim2() != 1 || matA.getDim2() != x.getDim1() || x.getDim2() != 1)
			throw Matrix<>::dim_exception("statistik: frobeniusnorm_b_minus_Ax: Dimensionen unpassend");
		int n = b.getDim1();
		int d = matA.getDim2();
		double sum = 0;
		for (int i = 0; i < n; i++) {
			double t = b.getEntry(i, 0);
			for (int j = 0; j < d; j++) {
				t -= matA.getEntry(i, j) * x.getEntry(j, 0);
			}
			sum += t*t;
		}
		if (!square)
			sum = sqrt(sum);
		return sum;
	}
}

Epsilon statistik::calcEps(const Analysed& exakt, const Analysed& skizze) {
	Epsilon erg;
	calcWinkel(exakt, skizze, erg);
	calcWasserstein(exakt, skizze, erg);

	erg.mean_dist = (exakt.mean - skizze.mean).frobeniusnorm(true);
	erg.err_exact = frobeniusnorm_b_minus_Ax(exakt.b, exakt.A, exakt.mean);
	erg.err_approx = frobeniusnorm_b_minus_Ax(skizze.b, skizze.A, skizze.mean);

	erg.eps1 = (erg.err_approx / erg.err_exact) -1;
	erg.eps2 = frobeniusnorm_b_minus_Ax(exakt.b, exakt.A, skizze.mean) / erg.err_exact -1;
	vector<double> ata_eigen = exakt.eigenValues.toVector();
	double sigma_min_a = 1/ata_eigen[0]; // kleinster Eigenwert von ATA;
	for (unsigned i = 0; i < ata_eigen.size(); i++) if (1/ata_eigen[i] < sigma_min_a) sigma_min_a = 1/ata_eigen[i];
	erg.eps3 = (exakt.mean - skizze.mean).frobeniusnorm() / erg.err_exact * sigma_min_a;
	erg.eps4 = (exakt.mean - skizze.mean).frobeniusnorm() / exakt.mean.frobeniusnorm() / 2;
	erg.eps5 = (erg.trace_var_exact + erg.trace_var_approx - 2*erg.trace_var_mix) / (erg.trace_var_exact);
	return erg;
}



void statistik::compare(const Analysed& exakt, const Analysed& skizze, const Matrix<double>& x) {
	Analysed beta;
	beta.A.set_noCopy(exakt.A);
	beta.b.set_noCopy(exakt.b);
	beta.var.set_noCopy(exakt.var);
	if (x.isEmpty())
		beta.mean = exakt.mean;
	else
		beta.mean = x;
	//long long t0 = clock();
	Epsilon results = calcEps(exakt, skizze);
	//long long time_eps = clock() - t0;


	// AUSGABE
/*		cout << "Berechnungsdauer: " << '\n';
		cout << "    Skizzengröße = " << skizze.A.getDim1() << '\n';
		cout << "    Reading = " << time2str(exakt.time_base) << '\n';
		cout << "    Skizze = " << time2str(skizze.time_base) << '\n';
		cout << "    Analyse (exakt) = " << time2str(exakt.time_analysis) << '\n';
		cout << "    Analyse (skizze) = " << time2str(skizze.time_analysis) << '\n';
		cout << "    Postprocessing = " << time2str(time_eps) << '\n';
		cout << "Varianz-Fehler (Wassersteinmetrik): " << '\n';
		cout << "    Avar <-> SAvar = " << (exakt.eigenValues - skizze.eigenValues).frobeniusnorm(true) << '\n';
		cout << "        Größenordnung: " << exakt.eigenValues.frobeniusnorm(true) << '\n';
		cout << "Varianz-Fehler (Winkelunterschied):" << '\n';
		cout << "    Greedy: " << '\n';
		cout << "        Vektor: " << results.angles_greedy.toVector();
		cout << "        Worst:  " << results.angles_greedy.map(fabs).fold(min, 1.0) << '\n';
		cout << "        Best:   " << results.angles_greedy.map(fabs).fold(max, 0.0) << '\n';
		cout << "        Mean:   " << results.angles_greedy.map(fabs).mean() << '\n';
		cout << "        Dist:   " << sqrt(skizze.A.getDim2() - results.angles_greedy.frobeniusnorm(true)) << '\n';
	//	cout << "    Original: " << '\n';
	//	cout << "        Vektor: " << results.winkel_original.toVector();
	//	cout << "        Worst:  " << results.winkel_original.map(fabs).fold(min, 1.0) << '\n';
	//	cout << "        Best:   " << results.winkel_original.map(fabs).fold(max, 0.0) << '\n';
	//	cout << "        Mean:   " << results.winkel_original.map(fabs).mean() << '\n';
	//	cout << "    Sortiert: " << '\n';
	//	cout << "        Vektor: " << results.winkel_sorted.toVector();
	//	cout << "        Worst:  " << results.winkel_sorted.map(fabs).fold(min, 1.0) << '\n';
	//	cout << "        Best:   " << results.winkel_sorted.map(fabs).fold(max, 0.0) << '\n';
	//	cout << "        Mean:   " << results.winkel_sorted.map(fabs).mean() << '\n';
		cout << "    Eigenwerte: " << '\n';
		cout << "        Exakt:  " << exakt.eigenValues.toVector();
		cout << "        Skizze: " << skizze.eigenValues.toVector();

		cout << "Erwartungs-Fehler (Wassersteinmetrik): " << '\n';
		cout << "    ||amean - samean||^2 = " << results.mean_dist << '\n';
		cout << "        Größenordnung: " << exakt.mean.frobeniusnorm(true) << '\n';
		cout << "    tr(M1) = " << results.trace_var_exact << '\n';
		cout << "    tr(M2) = " << results.trace_var_approx << '\n';
		cout << "    tr(mix) = " << results.trace_var_mix << '\n';
		cout << "    tr_sum = " << results.trace_sum << '\n';
		cout << "    sum = " << results.mean_dist + results.trace_sum << '\n';

	if (!x.isEmpty()) {
		cout << "    ||amean - xmean||^2 = " << (exakt.mean - beta.mean).frobeniusnorm(true) << '\n';
		cout << "    ||samean - xmean||^2 = " << (skizze.mean - beta.mean).frobeniusnorm(true) << '\n';
	}
		cout << "Sonstige Bewertungen: " << '\n';
		cout << "    Z = ||b - A*amean|| = " << results.err_exact << '\n';
		cout << "    ZS = ||Sb - SA*samean|| = " << results.err_approx << '\n';
		cout << "    eps1 = ZS / Z - 1 = " << results.eps1 << '\n';
		cout << "    eps2 = ||b - A*samean|| / Z - 1 = " << results.eps2 << '\n';
		cout << "    eps3 = ||amean - samean|| / Z * sigma_min(A) = " << results.eps3 << '\n';
		cout << "    eps4 = ||amean - samean|| / 2*||amean|| = " << results.eps4 << '\n';
		cout << "    eps5 = (tr(M1) + tr(M2) - 2*tr(mix)) / (tr(M1)) = " << results.eps5 << '\n';
		cout << flush;*/
}

void statistik::run(const Readed& rd, const std::vector<double>& eps, const std::vector<std::string>& sa_path, bool /*printSA*/) {
	//int n = rd.A.getDim1();
	//int d = rd.A.getDim2();
	/*cout << "=== STATISTIK ===" << '\n';
	cout << rd.name << '\n';;
	cout << n << " x " << d << '\n';
	cout << "eps = " << eps;
	cout << '\n';*/

	Analysed exakt = analyse(rd.A, rd.b);
	for (unsigned i = 0; i < eps.size(); i++) {
		//int s = (int) ceil(d * log(d) / (eps[i] * eps[i]));
		/*cout << ">>> epsilon = " << eps[i] << '\n';
		cout << ">>> " << n << " x " << d << " -> " << s << " x " << d << endl;*/
		Base sa = embed<BCH_Sketch>(rd, eps[i]); //(rd.A, rd.b, eps[i], printSA);
		Analysed skizze = analyse(sa);
		compare(exakt, skizze, rd.x);
		if (i < sa_path.size()) {
			string s(sa_path[i]);
			write(sa, (s + ".txt").data());
			exakt.mean.writeR((s + "_Amean.txt").data());
			skizze.mean.writeR((s + "_SAmean.txt").data());
		}
		//cout << endl;
	}
}

statistik::WeightedSampler::WeightedSampler(int k) :
	currentIndex_(0),
	weightSum_(0),
	choosen_(k) {

}

statistik::WeightedSampler::WeightedSampler(double eps, int d) :
    WeightedSampler(ceil(d * log(d) / (eps * eps))) {

}

void statistik::WeightedSampler::nextWeight(double weight) {
	weightSum_ += weight;
	double p = weight / weightSum_;
	for (unsigned i = 0; i < choosen_.size(); i++)
		if (useful::rnd() < p) {
			choosen_[i] = Entry(currentIndex_, weight);
		}
	currentIndex_++;
}

void statistik::WeightedSampler::nextWeights(const std::vector<double>& weights) {
	for (double w: weights)
		nextWeight(w);
}

std::vector<statistik::WeightedSampler::Entry> statistik::WeightedSampler::getResult() const {
	return choosen_;
}

Base statistik::WeightedSampler::getSampledMatrix(const Base& base, long long time) const {
	long long t0 = clock();
	vector<Entry> v = getResult();
	int d = base.A.getDim2();
	Base erg;
	erg.A = Matrix<>(v.size(), d);
	erg.b = Matrix<>(v.size(), 1);
	for (unsigned i = 0; i < v.size(); i++) {
		const Entry e = v[i];
		if (e.index >= 0) {
			double norm = 1 / sqrt(v.size() * e.weight / weightSum_);
			for (int j = 0; j < d; j++)
				erg.A(i, j) = base.A(e.index, j) * norm;
			erg.b(i, 0) = base.b(e.index, 0) * norm;
		} else {
			for (int j = 0; j < d; j++)
				erg.A(i, j) = 0;
			erg.b(i, 0) = 0;
		}
	}
	erg.time = time + clock() - t0;
	return erg;
}


// Quelle: "Fast approximation of matrix coherence and statistical leverage"
Matrix<double> statistik::underConstrainedRegression(const Matrix<double>& matA, const Matrix<double>& matb, std::vector<double> levScores, double eps) {
	// minimize ||Ax - b||
	// A: n x d, x: d x 1, b: n x 1

	int n = matA.getDim1();
	int d = matA.getDim2();

	if (matb.getDim1() != n || matb.getDim2() != 1)
		throw Matrix<>::dim_exception("statistik::underConstraintRegression: matb");
	if (levScores.size() != (unsigned) d)
		throw Matrix<>::dim_exception("statistik::underConstraintRegression: levScores");

	// Leverage Scores normalisieren
	double levSum = 0;
	for (double v: levScores)
		levSum += v;
	for (int i = 0; i < d; i++)
		levScores[i] /= levSum;

	// Leverage Scores kumulieren
	std::vector<double> levKum(d);
	levKum[0] = levScores[0];
	for (int i = 1; i < d; i++)
		levKum[i] = levKum[i-1] + levScores[i];

	// Zieldimension (d -> r)
    int r = (int) ceil(n / (eps * eps) * log(n / (eps * eps)));

	// in S(d x r) steht in jeder Spalte genau eine Zahl

	// Einbettung
	Matrix<double> matAS(n, r);
	for (int j = 0; j < r; j++) {
		double rnd = useful::rnd();
		// TODO: binäre Suche wäre besser...

		int select_idx = 0;
		while (select_idx < d && levKum[select_idx] < rnd)
			select_idx++;

		for (int i = 0; i < n; i++) {
			matAS(i, j) = matA(i, select_idx) / sqrt(r * levScores[select_idx]);
		}

	}

	Matrix<double> xopt = matA.getTranspose() * (matAS.pseudo_inv().calcATA() * matb);
	return xopt;
}


