/** @file BCH.cpp
 *  @brief Implementation of BCH.h
 *  @author Jens Quedenfeld
 */

#include "sketch/BCH.h"
#include "util/Useful.h"

typedef unsigned long long ULL;

BCH::BCH(int deg) :
	deg_(deg),
	seed_(deg / 2) {
	if (deg % 2 == 0)
		s0_ = 0;
	else
		s0_ = useful::rand_() % 2;
	for (unsigned i = 0; i < seed_.size(); i++) {
		seed_[i] = useful::random_LL();
	}
}

BCH::BCH(std::vector<ULL> seed, int s0) {
	std::swap(seed, seed_);
	deg_ = seed_.size() / 2;
	if (s0 == EVEN_DEGREE) {
		s0_ = 0;
	} else {
		s0_ = s0 % 2;
		deg_++;
	}
}

BCH::~BCH() {
	// nothing to do
}



int BCH::getDegree() const {
	return deg_;
}

std::vector<ULL> BCH::getSeed() const {
	return seed_;
}

int BCH::get_s0() const {
	return s0_;
}



bool BCH::operator==(BCH& bch) const {
	return deg_ == bch.deg_
		&& seed_ == bch.seed_
		&& s0_ == bch.s0_;
}
