#ifndef TIMER_H
#define TIMER_H

#include <chrono>
#include <iostream>
#include <string>

template <class Measurement = std::chrono::milliseconds>
class Timer {
public:
    typedef std::chrono::high_resolution_clock::time_point TimePoint;
private:
    TimePoint timeStart;
    Measurement duration;
    bool running;
public:
    Timer() :
        duration(0) {
        startTimer();
    }

    TimePoint now() const {
        return std::chrono::high_resolution_clock::now();
    }

    void startTimer() {
        running = true;
        timeStart = now();
    }

    void stopTimer() {
        TimePoint timeNow = now();
        duration += std::chrono::duration_cast<Measurement>(timeNow - timeStart);
        running = false;
    }

    void resetTimer() {
        duration = Measurement(0);
        startTimer();
    }

    Measurement getDuration() {
        if (running) {
            TimePoint timeNow = now();
            duration += std::chrono::duration_cast<Measurement>(timeNow - timeStart);
            timeStart = timeNow;
        }
        return duration;
    }

};

template <class Measurement>
std::ostream& operator<<(std::ostream& o, Timer<Measurement>& timer) {
    o << timer.getDuration().count();
    return o;
}

#endif // TIMER_H
