/** @file Useful.cpp
 *  @brief Implementation of Useful.h
 *  @author Jens Quedenfeld
 */

#define _USE_MATH_DEFINES

#include "util/Useful.h"
#include "mat/matrix.h"
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <ctime>
#include <iostream>
#include <sstream>
#include <string>

#ifndef M_PI
#define M_PI 3.1415926535
#endif

using namespace std;

long long useful::timeMultiply_naive = 0;
long long useful::timeMultiply_other = 0;
long long useful::timeMultiply_add = 0;
long long useful::timeMultiply_last = 0;

bool useful::debug_ = false;


long long useful::getTime(long long& t0) {
	long long t1 = clock();
	long long diff = t1 - t0;
	t0 = t1;
	return diff;
}

void useful::printTime(const char* prefix, long long &t0) {
	long long t1 = clock();
	//cout << prefix << time2str(t1 - t0) << endl;
	t0 = t1;
}

string useful::time2str(long long time) {
	// Umwandlung in ms
	time = time * 1000 / CLOCKS_PER_SEC;

	int ms = time % 1000;
	time /= 1000;
	int sec = time % 60;
	time /= 60;
	int min = time % 60;
	time /= 60;
	int h = time % 24;
	time /= 24;
	int d = time;

	const int size = 255;
	char buffer[size];
	if (d != 0) snprintf (buffer, (size_t) size, "%dd %dh %dmin", d, h, min);
	else if (h != 0) snprintf (buffer, (size_t) size, "%dh %dmin %ds", h, min, sec);
	else if (min != 0) snprintf (buffer, (size_t) size, "%dmin %ds %dms", min, sec, ms);
	else snprintf (buffer, (size_t) size, "%ds %dms", sec, ms);

	string s = string(buffer);
	return s;
}

const char* useful::b2s(bool b) {
	return b ? "true" : "false";
}

namespace {
    std::mt19937_64 rnd_generator(time(0));
}

unsigned long long useful::rand_() {
    return rnd_generator();
}

void useful::srand_(unsigned seed) {
    rnd_generator.seed(seed);
}

double useful::rnd() {
	return ((double) random_LL() / RANDOM_LL_MAX);
}

double useful::rnd(double min, double max) {
	return rnd() * (max - min) + min;
}

unsigned long long useful::random_LL() {
    return rand_();
}

long long useful::random_LL(long long min, long long max) {
	unsigned long long size = max - min;
	unsigned long long rnd;
	do {
		rnd = random_LL();
	} while (rnd > RANDOM_LL_MAX - RANDOM_LL_MAX % size);
	return min + rnd % size;
}

double useful::rnd_boxMuller(double sigma) {
    double u1 = (rand_()+1.) / (RANDOM_LL_MAX+1.); // 1..32768 / 32768 --> 0 < x <= 1
    double u2 = (rand_()+1.) / (RANDOM_LL_MAX+1.);
	double z1 = sqrt(-2*log(u1)) * cos(2*M_PI*u2);
	return z1*sigma;
}

void useful::err(const char* str) {
	//cerr << str << endl;
	throw std::runtime_error(std::string(str));
}

std::vector<std::string> useful::split(std::string str, char c) {
	std::vector<std::string> v;
	unsigned begin = 0;
	for (unsigned i = 0; i < str.length(); i++) {
		if (str.data()[i] == c) {
			v.push_back(str.substr(begin, i - begin));
			begin = i+1;
		}
	}
	if (str.length() != 0)
		v.push_back(str.substr(begin, str.length() - begin));
	return v;
}

Matrix<double> useful::randomDoubleMatrix(int d1, int d2, double min, double max) {
	Matrix <double> m = Matrix <double> (d1, d2);
	for (int i = 0; i < d1; i++)
		for (int j = 0; j < d2; j++) {
			m.updateEntry(i, j, random_LL() * (max - min) / RANDOM_LL_MAX + min);
		}

	return m;
}

Matrix<int> useful::randomIntMatrix(int d1, int d2, int min, int max) {
	Matrix <int> m = Matrix <int> (d1, d2);
	for (int i = 0; i < d1; i++)
		for (int j = 0; j < d2; j++) {
			m.updateEntry(i, j, random_LL() % (max - min + 1) + min);
		}

	return m;
}

bool useful::fileExists(std::string path) {
	bool b = true;
	fstream f;
	f.open(path.c_str(), ios::in);
	if (!f) b = false;
	f.close();
	return b;
}

int useful::sequentialXor(unsigned long long x, int counter){
	//Berechnet sequentielles XOR für eine Bitfolge modulo 2
	while (x > 0){
		counter = counter ^ 1;
		x = x & (x-1);
	}
	return counter;
}


void useful::assert_failed(const char* msg, const char* expr, const char* file, int line) {
	/*using std::cerr; using std::endl;
	cerr << "Assertion failed!\n" << endl;
	cerr << "File: " << file << ", Line " << line << '\n';
	cerr << "Expression: " << expr << '\n';
	cerr << "Message: " << msg << '\n';*/
    throw std::runtime_error("assertion failed");
}
